/***************************************************************************
 * e2cp.c - Copy an ext2 file
 *
 * Copyright (C) 1995 Claus Tondering, ct@login.dknet.dk
 * Copyright (C) 1997 Peeter Joot
 *
 * This file may be redistributed under the terms of the GNU Public License.
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <linux/types.h>
#include "linux/ext2_fs.h"
#include "linux/stat.h"

#include "ext2fs/ext2fs.h"
#include "ext2fs/ext2_err.h"

#include "dev.h"
#include "e2err.h"
#include "getopt.h"

extern io_manager nt_io_manager;

char omode[] = "wb";
FILE *ofile;
char *blockbuf;

#define MIN(a,b) ((a)<(b) ? (a) : (b))

/**********************************************************************
 * usage prints usage information and exits
 **********************************************************************/

void
usage()
{
	fprintf(stderr, 
"SYNOPSIS\n"
"\tE2CP [-bth] file1 file2\n"
"\n"
"DESCRIPTION\n"
"\tThe E2CP program will copy the contents of the file identified\n"
"\tby ext2 pathname 'file1' to the Native file 'file2'.\n"
"\n"
"\tThe following options are available:\n"
"\n"
"-b\tBinary mode (default). No translation performed.\n"
"\n"
"-t\tText mode. LF translated to CR/LF.\n"
"-h\tPrints this message.\n"
);
	exit(1);
}

/**********************************************************************
 * myproc is called once for every block in the file
 **********************************************************************/

static int myproc(ext2_filsys fs,
				  blk_t *blocknr,
				  int blockcnt,
				  void *private)
{
	int err;
	struct ext2_inode *ino = private;
	static nextblock = 0;

	if (blockcnt < 0)			/* We're reading an inderect block */
		return 0;

	if (nextblock+1 < blockcnt)
		memset(blockbuf, 0, fs->blocksize);

	while (nextblock++ < blockcnt)
		if (fwrite(blockbuf, fs->blocksize, 1, ofile) < 0) {
			fprintf(stderr,"Cannot write destination file\n");
			return BLOCK_ABORT;
		}

	err = io_channel_read_blk(fs->io, *blocknr, 1, blockbuf);
	if (err) {
		fprintf(stderr,"Cannot read source file\n");
		return BLOCK_ABORT;
	}

	if (fwrite(blockbuf, MIN(fs->blocksize, ino->i_size - blockcnt*fs->blocksize), 1, ofile) < 0) {
		fprintf(stderr,"Cannot write destination file\n");
		return BLOCK_ABORT;
	}
	
	return 0;
}


/**********************************************************************
 * main routine
 **********************************************************************/

char * prog_name;

main(int argc, char **argv)
{
	int err, c;
	char *src, *dst;
	ext2_filsys fs;
	ino_t ino;
	struct ext2_inode e2ino;
	e2cwd_t cwd;

	prog_name = argv[0];
	initialize_ext2_error_table();

	opterr = 0;
	while ((c=getopt(argc, argv, "bth")) != -1) {
		switch (c) {
			case 'b': 
			case 't': 
				omode[1] = c;
				break;
			case 'h':
			case '?': 
			default :
				usage();
		}
	}

	if (argc!=optind+2)
		usage();

	src = argv[optind];
	dst = argv[optind+1];

	err = cwdenv_lookup(&cwd);
	if (err)
		e2_err(err,0);
	
	/* Open file system */
	err = ext2fs_open((const char *)&cwd, 0, 0, 0, nt_io_manager, &fs);
	if (err)
		e2_err(EXT2_ET_CANT_OPEN_FS,err);

	/* Allocate file buffer */
	blockbuf = malloc(fs->blocksize);
	if (!blockbuf) {
		fprintf(stderr,"Cannot malloc buffer\n");
		exit(1);
	}

	/* Lookup specified name */
	err = ext2fs_namei(fs, 2, cwd.cwdino, src, &ino);
	if (err)
		e2_err(EXT2_ET_NO_SRC_FILE,err);

	/* Read specified inode */
	err = ext2fs_read_inode(fs, ino, &e2ino);
	if (err)
		e2_err(EXT2_ET_CANT_READ_IN_INFO,err);

	/* Is it a regular file? */
	if (!S_ISREG(e2ino.i_mode)) {
		fprintf(stderr,"%s is not a regular file\n",src);
		exit(1);
	}

	ofile=fopen(dst,omode);
	if (!ofile) {
		fprintf(stderr,"Cannot create %s\n",dst);
		exit(1);
	}

	err = ext2fs_block_iterate(fs, ino, 0, 0, myproc, &e2ino);
	if (err)
		e2_err(EXT2_ET_CANT_READ_SRC_FILE,err);

	return 0;
}
